@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
 from functools import partial
+from typing import Callable, Iterable
 
 import django
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models import Manager, ForeignKey, OneToOneField, options
+from django.db.models import ForeignKey, Manager, ManyToManyField, OneToOneField, options
 from django.db.models.base import ModelBase
 from django.db.models.signals import post_init
 from django.utils.functional import cached_property
@@ -11,17 +11,19 @@ from django.utils.functional import cached_property
 from modeltranslation import settings as mt_settings
 from modeltranslation.fields import (
     NONE,
-    create_translation_field,
-    TranslationFieldDescriptor,
-    TranslatedRelationIdDescriptor,
     LanguageCacheSingleObjectDescriptor,
+    TranslatedManyToManyDescriptor,
+    TranslatedRelationIdDescriptor,
+    TranslationFieldDescriptor,
+    create_translation_field,
 )
 from modeltranslation.manager import (
     MultilingualManager,
     MultilingualQuerysetManager,
-    rewrite_lookup_key,
     append_translated,
+    rewrite_lookup_key,
 )
+from modeltranslation.thread_context import auto_populate_mode
 from modeltranslation.utils import build_localized_fieldname, parse_field
 
 
@@ -48,7 +50,7 @@ class FieldsAggregationMetaClass(type):
             if isinstance(base, FieldsAggregationMetaClass):
                 attrs['fields'].update(base.fields)
         attrs['fields'] = tuple(attrs['fields'])
-        return super(FieldsAggregationMetaClass, cls).__new__(cls, name, bases, attrs)
+        return super().__new__(cls, name, bases, attrs)
 
 
 class TranslationOptions(metaclass=FieldsAggregationMetaClass):
@@ -81,8 +83,8 @@ class TranslationOptions(metaclass=FieldsAggregationMetaClass):
         self.model = model
         self.registered = False
         self.related = False
-        self.local_fields = dict((f, set()) for f in self.fields)
-        self.fields = dict((f, set()) for f in self.fields)
+        self.local_fields = {f: set() for f in self.fields}
+        self.fields = {f: set() for f in self.fields}
         self.related_fields = []
 
     def validate(self):
@@ -139,7 +141,7 @@ class TranslationOptions(metaclass=FieldsAggregationMetaClass):
 class MultilingualOptions(options.Options):
     @cached_property
     def base_manager(self):
-        manager = super(MultilingualOptions, self).base_manager
+        manager = super().base_manager
         patch_manager_class(manager)
         return manager
 
@@ -389,7 +391,7 @@ def populate_translation_fields(sender, kwargs):
     defined (for example to make lookups / filtering without resorting to
     query fallbacks).
     """
-    populate = mt_settings.AUTO_POPULATE
+    populate = auto_populate_mode()
     if not populate:
         return
     if populate is True:
@@ -433,7 +435,7 @@ def patch_related_object_descriptor_caching(ro_descriptor):
     ro_descriptor.__class__ = NewSingleObjectDescriptor
 
 
-class Translator(object):
+class Translator:
     """
     A Translator object encapsulates an instance of a translator. Models are
     registered with the Translator using the register() method.
@@ -442,6 +444,8 @@ class Translator(object):
     def __init__(self):
         # All seen models (model class -> ``TranslationOptions`` instance).
         self._registry = {}
+        # List of funcs to execute after all imports are done.
+        self._lazy_operations: Iterable[Callable] = []
 
     def register(self, model_or_iterable, opts_class=None, **options):
         """
@@ -542,11 +546,16 @@ class Translator(object):
                 fallback_undefined=field_fallback_undefined,
             )
             setattr(model, field_name, descriptor)
-            if isinstance(field, ForeignKey):
+            if isinstance(field, (ForeignKey, ManyToManyField)):
                 # We need to use a special descriptor so that
                 # _id fields on translated ForeignKeys work
                 # as expected.
-                desc = TranslatedRelationIdDescriptor(field_name, model_fallback_languages)
+                desc_class = (
+                    TranslatedManyToManyDescriptor
+                    if isinstance(field, ManyToManyField)
+                    else TranslatedRelationIdDescriptor
+                )
+                desc = desc_class(field_name, model_fallback_languages)
                 setattr(model, field.get_attname(), desc)
 
                 # Set related field names on other model
@@ -643,6 +652,13 @@ class Translator(object):
             )
         return opts
 
+    def execute_lazy_operations(self) -> None:
+        while self._lazy_operations:
+            self._lazy_operations.pop(0)(translator=self)
+
+    def lazy_operation(self, func: Callable, *args, **kwargs) -> None:
+        self._lazy_operations.append(partial(func, *args, **kwargs))
+
 
 # This global object represents the singleton translator object
 translator = Translator()
